package org.nakedobjects.nof.persist.objectstore.inmemory;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.persist.SimpleOidGenerator;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;
import org.nakedobjects.nof.persist.AbstractObjectPersistorInstaller;
import org.nakedobjects.nof.persist.DefaultPersistAlgorithm;
import org.nakedobjects.nof.persist.objectstore.ObjectStorePersistor;


/**
 * Installs the transient object store.
 */
public class TransientPersistorInstaller extends AbstractObjectPersistorInstaller {
    private static final Logger LOG = Logger.getLogger(TransientPersistorInstaller.class);
    private TransientObjectStore transientObjectStore;

    public NakedObjectPersistor createObjectPersistor(final NakedObjectConfiguration configuration) {
        LOG.info("installing " + this.getClass().getName());

        if (transientObjectStore != null) {
            throw new NakedObjectRuntimeException(
                    "Attempting to create persistor for second context; transient persistence only works for single user systems");
        }

        transientObjectStore = new TransientObjectStore();
        ObjectStorePersistor persistor = new ObjectStorePersistor();
        persistor.setObjectStore(transientObjectStore);
        persistor.setPersistAlgorithm(new DefaultPersistAlgorithm());
        persistor.setCheckObjectsForDirtyFlag(true);
        persistor.setOidGenerator(new SimpleOidGenerator());
        return persistor;
    }

    public String getName() {
        return "transient";
    }
}
// Copyright (c) Naked Objects Group Ltd.
